@@ -646,33 +646,47 @@ SCA_ <- function(x = 1, Data, AddInd = "B", SR = c("BH", "Ricker", "none"),
       SE_Dev[is.na(SE_Dev)] <- 0
     }
     
-    ref_pt <- lapply(seq_len(ifelse(tv_M == "walk", n_y, 1)), ref_pt_SCA, obj = obj, report = report)
-    if(catch_eq == "Baranov") {
-      report$FMSY <- vapply(ref_pt, getElement, numeric(1), "FMSY")
-    } else {
-      report$UMSY <- vapply(ref_pt, getElement, numeric(1), "UMSY")
-    }
-    report$MSY <- vapply(ref_pt, getElement, numeric(1), "MSY")
-    report$VBMSY <- vapply(ref_pt, getElement, numeric(1), "VBMSY")
-    report$RMSY <- vapply(ref_pt, getElement, numeric(1), "RMSY")
-    report$BMSY <- vapply(ref_pt, getElement, numeric(1), "BMSY")
-    report$EMSY <- vapply(ref_pt, getElement, numeric(1), "EMSY")
-    
     refyear <- eval(refyear)
+    
+    ref_pt <- ref_pt_SCA(y = refyear, obj = obj, report = report)
     if(catch_eq == "Baranov") {
-      Assessment@FMSY <- report$FMSY[refyear]
-      Assessment@F_FMSY <- structure(report$F/report$FMSY[refyear], names = Year)
+      report$FMSY <- ref_pt$FMSY
     } else {
-      Assessment@UMSY <- report$UMSY[refyear]
-      Assessment@U_UMSY <- structure(report$U/report$UMSY[refyear], names = Year)
+      report$UMSY <- ref_pt$UMSY
     }
-    Assessment@MSY <- report$MSY[refyear]
-    Assessment@BMSY <- report$BMSY[refyear]
-    Assessment@SSBMSY <- report$EMSY[refyear]
-    Assessment@VBMSY <- report$VBMSY[refyear]
-    Assessment@B_BMSY <- structure(report$B/report$BMSY[refyear], names = Yearplusone)
-    Assessment@SSB_SSBMSY <- structure(report$E/report$EMSY[refyear], names = Yearplusone)
-    Assessment@VB_VBMSY <- structure(report$VB/report$VBMSY[refyear], names = Yearplusone)
+    report$MSY <- ref_pt$MSY
+    report$VBMSY <- ref_pt$VBMSY
+    report$RMSY <- ref_pt$RMSY
+    report$BMSY <- ref_pt$BMSY
+    report$EMSY <- ref_pt$EMSY
+    report$refyear <- refyear
+    
+    if(!all(refyear == 1)) { # New reference points based on change in M
+      report$new_B0 <- Assessment@B0 <- ref_pt$new_B0
+      report$new_E0 <- Assessment@SSB0 <- ref_pt$new_E0
+      report$new_VB0 <- Assessment@VB0 <- ref_pt$new_VB0
+      report$new_R0 <- Assessment@R0 <- ref_pt$new_R0
+      report$new_h <- Assessment@h <- ref_pt$new_h
+      
+      Assessment@B_B0 <- Assessment@B/Assessment@B0
+      Assessment@SSB_SSB0 <- Assessment@SSB/Assessment@SSB0
+      Assessment@VB_VB0 <- Assessment@VB/Assessment@VB0
+    }
+    
+    if(catch_eq == "Baranov") {
+      Assessment@FMSY <- report$FMSY
+      Assessment@F_FMSY <- structure(report$F/Assessment@FMSY, names = Year)
+    } else {
+      Assessment@UMSY <- report$UMSY
+      Assessment@U_UMSY <- structure(report$U/Assessment@UMSY, names = Year)
+    }
+    Assessment@MSY <- report$MSY
+    Assessment@BMSY <- report$BMSY
+    Assessment@SSBMSY <- report$EMSY
+    Assessment@VBMSY <- report$VBMSY
+    Assessment@B_BMSY <- structure(report$B/Assessment@BMSY, names = Yearplusone)
+    Assessment@SSB_SSBMSY <- structure(report$E/Assessment@SSBMSY, names = Yearplusone)
+    Assessment@VB_VBMSY <- structure(report$VB/Assessment@VBMSY, names = Yearplusone)
     Assessment@Dev <- Dev
     Assessment@SE_Dev <- SE_Dev
     Assessment@TMB_report <- report
@@ -682,7 +696,7 @@ SCA_ <- function(x = 1, Data, AddInd = "B", SR = c("BH", "Ricker", "none"),
                      obs_error = list(array(1, c(1, 1, nsurvey)), matrix(1, 1, 1)),
                      process_error = matrix(1, 1, 1)) %>% slot("Catch") %>% as.vector()
     }
-    Assessment@forecast <- list(per_recruit = ref_pt[[refyear]][[7]], catch_eq = catch_eq_fn)
+    Assessment@forecast <- list(per_recruit = ref_pt[["per_recruit"]], catch_eq = catch_eq_fn)
   }
   return(Assessment)
 }
@@ -706,7 +720,7 @@ ref_pt_SCA <- function(y = 1, obj, report) {
     Brec <- report$Brec
   }
   
-  M <- report$M[y, ]
+  M <- apply(report$M[y, , drop = FALSE], 2, mean)
   weight <- obj$env$data$weight
   mat <- obj$env$data$mat
   vul <- report$vul
@@ -746,12 +760,24 @@ ref_pt_SCA <- function(y = 1, obj, report) {
   EPR <- vapply(yield, getElement, numeric(1), "EPR")
   YPR <- vapply(yield, getElement, numeric(1), "YPR")
   
+  new_B0 <- yield[[1]]["B"] # New due to change in M
+  new_E0 <- yield[[1]]["E"]
+  new_VB0 <- yield[[1]]["VB"]
+  new_R0 <- yield[[1]]["R"]
+  if(SR == "BH") {
+    new_h <- Arec * EPR[1]/ (4 + Arec * EPR[1])
+  } else {
+    new_h <- 0.2 * (Arec * EPR[1])^0.8
+  }
+  
   if(catch_eq == "Baranov") {
     return(list(FMSY = FMSY, MSY = MSY, VBMSY = VBMSY, RMSY = RMSY, BMSY = BMSY, EMSY = EMSY,
-                per_recruit = data.frame(FM = Fvec, SPR = EPR/EPR[1], YPR = YPR), SR_par = SR_par))
+                per_recruit = data.frame(FM = Fvec, SPR = EPR/EPR[1], YPR = YPR), SR_par = SR_par,
+                new_B0 = new_B0, new_E0 = new_B0, new_VB0 = new_VB0, new_R0 = new_R0, new_h = new_h))
   } else {
     return(list(UMSY = UMSY, MSY = MSY, VBMSY = VBMSY, RMSY = RMSY, BMSY = BMSY, EMSY = EMSY,
-                per_recruit = data.frame(U = Fvec, SPR = EPR/EPR[1], YPR = YPR), SR_par = SR_par))
+                per_recruit = data.frame(U = Fvec, SPR = EPR/EPR[1], YPR = YPR), SR_par = SR_par,
+                new_B0 = new_B0, new_E0 = new_B0, new_VB0 = new_VB0, new_R0 = new_R0, new_h = new_h))
   }
   
 }
